@@ -131,8 +131,8 @@ static inline size_t flex_size(size_t align, size_t offset, size_t size, size_t
  * @return
  *         The allocated memory, or NULL on failure.
  */
-_malloc(free, 1)
-_aligned_alloc(1, 2)
+[[_malloc(free, 1)]]
+[[_aligned_alloc(1, 2)]]
 void *alloc(size_t align, size_t size);
 
 /**
@@ -145,8 +145,8 @@ void *alloc(size_t align, size_t size);
  * @return
  *         The allocated memory, or NULL on failure.
  */
-_malloc(free, 1)
-_aligned_alloc(1, 2)
+[[_malloc(free, 1)]]
+[[_aligned_alloc(1, 2)]]
 void *zalloc(size_t align, size_t size);
 
 /** Allocate memory for the given type. */
@@ -187,8 +187,8 @@ void *zalloc(size_t align, size_t size);
  * @return
  *         The reallocated memory, or NULL on failure.
  */
-_aligned_alloc(2, 4)
-_nodiscard
+[[_nodiscard]]
+[[_aligned_alloc(2, 4)]]
 void *xrealloc(void *ptr, size_t align, size_t old_size, size_t new_size);
 
 /** Reallocate memory for an array. */
@@ -214,7 +214,7 @@ void *xrealloc(void *ptr, size_t align, size_t old_size, size_t new_size);
  *         for (count + 1) elements.  On failure, errno will be non-zero, and
  *         ptr will returned unchanged.
  */
-_nodiscard
+[[_nodiscard]]
 void *reserve(void *ptr, size_t align, size_t size, size_t count);
 
 /**
@@ -272,7 +272,7 @@ void arena_free(struct arena *arena, void *ptr);
 /**
  * Allocate an object out of the arena.
  */
-_malloc(arena_free, 2)
+[[_malloc(arena_free, 2)]]
 void *arena_alloc(struct arena *arena);
 
 /**
@@ -353,7 +353,7 @@ void varena_free(struct varena *varena, void *ptr, size_t count);
  * @return
  *         The allocated struct, or NULL on failure.
  */
-_malloc(varena_free, 2)
+[[_malloc(varena_free, 2)]]
 void *varena_alloc(struct varena *varena, size_t count);
 
 /**
@@ -370,7 +370,7 @@ void *varena_alloc(struct varena *varena, size_t count);
  * @return
  *         The resized struct, or NULL on failure.
  */
-_nodiscard
+[[_nodiscard]]
 void *varena_realloc(struct varena *varena, void *ptr, size_t old_count, size_t new_count);
 
 /**
@@ -385,7 +385,7 @@ void *varena_realloc(struct varena *varena, void *ptr, size_t old_count, size_t
  * @return
  *         The resized struct, or NULL on failure.
  */
-_nodiscard
+[[_nodiscard]]
 void *varena_grow(struct varena *varena, void *ptr, size_t *count);
 
 /**
