@@ -17,15 +17,18 @@
 #include <locale.h>
 #include <nl_types.h>
 #include <pthread.h>
+#include <sched.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/ioctl.h>
 #include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <termios.h>
 #include <unistd.h>
 #include <wchar.h>
 
@@ -186,16 +189,6 @@ char *xgetdelim(FILE *file, char delim) {
 	}
 }
 
-int open_cterm(int flags) {
-	char path[L_ctermid];
-	if (ctermid(path) == NULL || strlen(path) == 0) {
-		errno = ENOTTY;
-		return -1;
-	}
-
-	return open(path, flags);
-}
-
 const char *xgetprogname(void) {
 	const char *cmd = NULL;
 #if BFS_HAS_GETPROGNAME
@@ -242,6 +235,36 @@ static int xstrtox_epilogue(const char *str, char **end, char *endp) {
 	return 0;
 }
 
+int xstrtos(const char *str, char **end, int base, short *value) {
+	long n;
+	if (xstrtol(str, end, base, &n) != 0) {
+		return -1;
+	}
+
+	if (n < SHRT_MIN || n > SHRT_MAX) {
+		errno = ERANGE;
+		return -1;
+	}
+
+	*value = n;
+	return 0;
+}
+
+int xstrtoi(const char *str, char **end, int base, int *value) {
+	long n;
+	if (xstrtol(str, end, base, &n) != 0) {
+		return -1;
+	}
+
+	if (n < INT_MIN || n > INT_MAX) {
+		errno = ERANGE;
+		return -1;
+	}
+
+	*value = n;
+	return 0;
+}
+
 int xstrtol(const char *str, char **end, int base, long *value) {
 	if (xstrtox_prologue(str) != 0) {
 		return -1;
@@ -282,6 +305,70 @@ int xstrtod(const char *str, char **end, double *value) {
 	return xstrtox_epilogue(str, end, endp);
 }
 
+int xstrtous(const char *str, char **end, int base, unsigned short *value) {
+	unsigned long n;
+	if (xstrtoul(str, end, base, &n) != 0) {
+		return -1;
+	}
+
+	if (n > USHRT_MAX) {
+		errno = ERANGE;
+		return -1;
+	}
+
+	*value = n;
+	return 0;
+}
+
+int xstrtoui(const char *str, char **end, int base, unsigned int *value) {
+	unsigned long n;
+	if (xstrtoul(str, end, base, &n) != 0) {
+		return -1;
+	}
+
+	if (n > UINT_MAX) {
+		errno = ERANGE;
+		return -1;
+	}
+
+	*value = n;
+	return 0;
+}
+
+/** Common epilogue for xstrtou*() wrappers. */
+static int xstrtoux_epilogue(const char *str, char **end, char *endp) {
+	if (xstrtox_epilogue(str, end, endp) != 0) {
+		return -1;
+	}
+
+	if (str[0] == '-') {
+		errno = ERANGE;
+		return -1;
+	}
+
+	return 0;
+}
+
+int xstrtoul(const char *str, char **end, int base, unsigned long *value) {
+	if (xstrtox_prologue(str) != 0) {
+		return -1;
+	}
+
+	char *endp;
+	*value = strtoul(str, &endp, base);
+	return xstrtoux_epilogue(str, end, endp);
+}
+
+int xstrtoull(const char *str, char **end, int base, unsigned long long *value) {
+	if (xstrtox_prologue(str) != 0) {
+		return -1;
+	}
+
+	char *endp;
+	*value = strtoull(str, &endp, base);
+	return xstrtoux_epilogue(str, end, endp);
+}
+
 /** Compile and execute a regular expression for xrpmatch(). */
 static int xrpregex(nl_item item, const char *response) {
 	const char *pattern = nl_langinfo(item);
@@ -557,6 +644,32 @@ pid_t xwaitpid(pid_t pid, int *status, int flags) {
 	return ret;
 }
 
+int open_cterm(int flags) {
+	char path[L_ctermid];
+	if (ctermid(path) == NULL || strlen(path) == 0) {
+		errno = ENOTTY;
+		return -1;
+	}
+
+	return open(path, flags);
+}
+
+int xtcgetwinsize(int fd, struct winsize *ws) {
+#if BFS_HAS_TCGETWINSIZE
+	return tcgetwinsize(fd, ws);
+#else
+	return ioctl(fd, TIOCGWINSZ, ws);
+#endif
+}
+
+int xtcsetwinsize(int fd, const struct winsize *ws) {
+#if BFS_HAS_TCSETWINSIZE
+	return tcsetwinsize(fd, ws);
+#else
+	return ioctl(fd, TIOCSWINSZ, ws);
+#endif
+}
+
 int dup_cloexec(int fd) {
 #ifdef F_DUPFD_CLOEXEC
 	return fcntl(fd, F_DUPFD_CLOEXEC, 0);
@@ -775,6 +888,70 @@ long xsysconf(int name) {
 	return ret;
 }
 
+#if BFS_HAS_SCHED_GETAFFINITY
+/** Get the CPU count in an affinity mask of the given size. */
+static long bfs_sched_getaffinity(size_t size) {
+	cpu_set_t set, *pset = &set;
+
+	if (size > sizeof(set)) {
+		pset = malloc(size);
+		if (!pset) {
+			return -1;
+		}
+	}
+
+	long ret = -1;
+	if (sched_getaffinity(0, size, pset) == 0) {
+#  ifdef CPU_COUNT_S
+		ret = CPU_COUNT_S(size, pset);
+#  else
+		bfs_assert(size <= sizeof(set));
+		ret = CPU_COUNT(pset);
+# endif
+	}
+
+	if (pset != &set) {
+		free(pset);
+	}
+	return ret;
+}
+#endif
+
+long nproc(void) {
+	long ret = 0;
+
+#if BFS_HAS_SCHED_GETAFFINITY
+	size_t size = sizeof(cpu_set_t);
+	do {
+		ret = bfs_sched_getaffinity(size);
+
+#  ifdef CPU_COUNT_S
+		// On Linux, sched_getaffinity(2) says:
+		//
+		//     When working on systems with large kernel CPU affinity masks, one must
+		//     dynamically allocate the mask argument (see CPU_ALLOC(3)).  Currently,
+		//     the only way to do this is by probing for the size of the required mask
+		//     using sched_getaffinity() calls with increasing mask sizes (until the
+		//     call does not fail with the error EINVAL).
+		size *= 2;
+#  else
+		// No support for dynamically-sized CPU masks
+		break;
+#  endif
+	} while (ret < 0 && errno == EINVAL);
+#endif
+
+	if (ret < 1) {
+		ret = xsysconf(_SC_NPROCESSORS_ONLN);
+	}
+
+	if (ret < 1) {
+		ret = 1;
+	}
+
+	return ret;
+}
+
 size_t asciilen(const char *str) {
 	return asciinlen(str, strlen(str));
 }
