@@ -158,16 +158,6 @@ FILE *xfopen(const char *path, int flags);
  */
 char *xgetdelim(FILE *file, char delim);
 
-/**
- * Open the controlling terminal.
- *
- * @flags
- *         The open() flags.
- * @return
- *         An open file descriptor, or -1 on failure.
- */
-int open_cterm(int flags);
-
 // #include <stdlib.h>
 
 /**
@@ -178,6 +168,16 @@ int open_cterm(int flags);
  */
 const char *xgetprogname(void);
 
+/**
+ * Like xstrtol(), but for short.
+ */
+int xstrtos(const char *str, char **end, int base, short *value);
+
+/**
+ * Like xstrtol(), but for int.
+ */
+int xstrtoi(const char *str, char **end, int base, int *value);
+
 /**
  * Wrapper for strtol() that forbids leading spaces.
  */
@@ -188,6 +188,26 @@ int xstrtol(const char *str, char **end, int base, long *value);
  */
 int xstrtoll(const char *str, char **end, int base, long long *value);
 
+/**
+ * Like xstrtoul(), but for unsigned short.
+ */
+int xstrtous(const char *str, char **end, int base, unsigned short *value);
+
+/**
+ * Like xstrtoul(), but for unsigned int.
+ */
+int xstrtoui(const char *str, char **end, int base, unsigned int *value);
+
+/**
+ * Wrapper for strtoul() that forbids leading spaces, negatives.
+ */
+int xstrtoul(const char *str, char **end, int base, unsigned long *value);
+
+/**
+ * Wrapper for strtoull() that forbids leading spaces, negatives.
+ */
+int xstrtoull(const char *str, char **end, int base, unsigned long long *value);
+
 /**
  * Wrapper for strtof() that forbids leading spaces.
  */
@@ -342,6 +362,29 @@ int xminor(dev_t dev);
  */
 pid_t xwaitpid(pid_t pid, int *status, int flags);
 
+#include <sys/ioctl.h> // May be necessary for struct winsize
+#include <termios.h>
+
+/**
+ * Open the controlling terminal.
+ *
+ * @flags
+ *         The open() flags.
+ * @return
+ *         An open file descriptor, or -1 on failure.
+ */
+int open_cterm(int flags);
+
+/**
+ * tcgetwinsize()/ioctl(TIOCGWINSZ) wrapper.
+ */
+int xtcgetwinsize(int fd, struct winsize *ws);
+
+/**
+ * tcsetwinsize()/ioctl(TIOCSWINSZ) wrapper.
+ */
+int xtcsetwinsize(int fd, const struct winsize *ws);
+
 // #include <unistd.h>
 
 /**
@@ -463,6 +506,11 @@ long xsysconf(int name);
 #define sysoption(name) \
 	(_POSIX_##name == 0 ? xsysconf(_SC_##name) : _POSIX_##name)
 
+/**
+ * Get the number of CPU threads available to the current process.
+ */
+long nproc(void);
+
 #include <wchar.h>
 
 /**
