@@ -24,20 +24,6 @@
 #include <time.h>
 #include <unistd.h>
 
-/** Get the initial value for ctx->threads (-j). */
-static int bfs_nproc(void) {
-	long nproc = xsysconf(_SC_NPROCESSORS_ONLN);
-
-	if (nproc < 1) {
-		nproc = 1;
-	} else if (nproc > 8) {
-		// Not much speedup after 8 threads
-		nproc = 8;
-	}
-
-	return nproc;
-}
-
 struct bfs_ctx *bfs_ctx_new(void) {
 	struct bfs_ctx *ctx = ZALLOC(struct bfs_ctx);
 	if (!ctx) {
@@ -50,9 +36,14 @@ struct bfs_ctx *bfs_ctx_new(void) {
 	ctx->maxdepth = INT_MAX;
 	ctx->flags = BFTW_RECOVER;
 	ctx->strategy = BFTW_BFS;
-	ctx->threads = bfs_nproc();
 	ctx->optlevel = 3;
 
+	ctx->threads = nproc();
+	if (ctx->threads > 8) {
+		// Not much speedup after 8 threads
+		ctx->threads = 8;
+	}
+
 	trie_init(&ctx->files);
 
 	ctx->umask = umask(0);
@@ -295,6 +286,7 @@ int bfs_ctx_free(struct bfs_ctx *ctx) {
 		}
 		free(ctx->paths);
 
+		free(ctx->kinds);
 		free(ctx->argv);
 		free(ctx);
 	}
