@@ -1,7 +1,7 @@
 // Copyright © Tavian Barnes <tavianator@tavianator.com>
 // SPDX-License-Identifier: 0BSD
 
-#include "prelude.h"
+#include "bfs.h"
 
 const char bfs_version[] = {
 #include "version.i"
