@@ -1,2 +1,2 @@
 # Make sure -ignore_readdir_race doesn't suppress ENOENT at the root
-fail invoke_bfs basic/nonexistent -ignore_readdir_race
+! invoke_bfs basic/nonexistent -ignore_readdir_race
