@@ -1,3 +1,2 @@
-closefrom 4
-ulimit -n 16
+ulimit -n $((NOPENFD + 13))
 [ "$(invoke_bfs deep -printf '%u %g\n' | uniq)" = "$(id -un) $(id -gn)" ]
