@@ -11,6 +11,6 @@ mkfifo p1 p2
 } &
 
 # Write the `sh` PID to p1, then hang reading p2 until we're killed
-! invoke_bfs p1 -exec sh -c 'echo $$ >p1 && read -r _ <p2' {} + || fail
+! invoke_bfs p1 -exec bash -c 'echo $$ >p1 && read -r _ <p2' bash {} + || fail
 
-wait
+_wait
