@@ -8,6 +8,7 @@
 #ifndef BFS_TESTS_H
 #define BFS_TESTS_H
 
+#include "bfs.h"
 #include "bfstd.h"
 #include "diag.h"
 
@@ -44,27 +45,23 @@ bool bfs_check_impl(bool result);
 /**
  * Check a condition, logging a message on failure but continuing.
  */
-#define bfs_check(...) \
-	bfs_check_impl(bfs_check_(#__VA_ARGS__, __VA_ARGS__, "", ""))
+#define bfs_check(cond, ...) \
+	bfs_check_impl((cond) || (bfs_check_(#cond, __VA_ARGS__), false))
 
-#define bfs_check_(str, cond, format, ...) \
-	((cond) ? true : (bfs_diag( \
-		sizeof(format) > 1 \
-			? "%.0s" format "%s%s" \
-			: "Check failed: `%s`%s", \
-		str, __VA_ARGS__), false))
+#define bfs_check_(str, ...) \
+	BFS_VA_IF(__VA_ARGS__) \
+		(bfs_diag(__VA_ARGS__)) \
+		(bfs_diag("Check failed: `%s`", str))
 
 /**
  * Check a condition, logging the current error string on failure.
  */
-#define bfs_echeck(...) \
-	bfs_check_impl(bfs_echeck_(#__VA_ARGS__, __VA_ARGS__, "", errstr()))
+#define bfs_echeck(cond, ...) \
+	bfs_check_impl((cond) || (bfs_echeck_(#cond, __VA_ARGS__), false))
 
-#define bfs_echeck_(str, cond, format, ...) \
-	((cond) ? true : (bfs_diag( \
-		sizeof(format) > 1 \
-			? "%.0s" format "%s: %s" \
-			: "Check failed: `%s`: %s", \
-		str, __VA_ARGS__), false))
+#define bfs_echeck_(str, ...) \
+	BFS_VA_IF(__VA_ARGS__) \
+		(bfs_ediag(__VA_ARGS__)) \
+		(bfs_ediag("Check failed: `%s`", str))
 
 #endif // BFS_TESTS_H
